//===--------------------------------------------------------------*- C -*-===//
//
// EMUBT: Binary Translation using LLVM
// © Terma 2018, 2019
// Authors: Mattias Holm <maho@terma.com>
//
//===----------------------------------------------------------------------===//

#ifndef TEMU_BINARY_TRANSLATOR_H
#define TEMU_BINARY_TRANSLATOR_H
#include "temu-c/Support/Objsys.h"
#include <stdint.h>

typedef enum {
  teBTS_TranslatedInstructions,
  teBTS_ExecutedInstructions,
  teBTS_TranslatedBlocks,
  teBTS_ExecutedBlocks,
  teBTS_CodeSize,
} temu_BTStatID;

typedef struct {
  int (*translate)(void *Obj, uint64_t PA, unsigned NumInstrs);
  int (*translateBlock)(void *Obj, uint64_t PA);

  // Manually chain blocks
  int (*chainBlocks)(void *Obj, uint64_t SourceBlockPA, uint64_t TargetBlockPA, int Arm);

  int (*clearBlock)(void *Obj, uint64_t PA);
  int (*clearBlocksOnPage)(void *Obj, uint64_t PA);
  void (*enableStatistics)(void *Obj, temu_BTStatID ID);
  void (*disableStatistics)(void *Obj, temu_BTStatID ID);
  uint64_t (*getStatistics)(void *Obj, temu_BTStatID ID);
  void (*clearStatistics)(void *Obj, temu_BTStatID ID);

  void (*enableKeepIR)(void *Obj);
  void (*disableKeepIR)(void *Obj);

  const char* (*disassemble)(void *Obj, uint64_t Addr);
  const char* (*disassembleIR)(void *Obj, uint64_t Addr);
} temu_BTIface;
#define TEMU_BT_IFACE_TYPE "BTIface"
TEMU_IFACE_REFERENCE_TYPE(temu_BT);

#endif // ! TEMU_BINARY_TRANSLATOR_H
