//===--------------------------------------------------------------*- C -*-===//
//
// EMUBT: Binary Translation using LLVM
// © Terma 2018, 2019
// Authors: Mattias Holm <maho@terma.com>
//
//===----------------------------------------------------------------------===//

#ifndef TEMU_INSTRUMENTER_H
#define TEMU_INSTRUMENTER_H
#include "temu-c/Support/Objsys.h"
#include <stdint.h>

typedef enum {
  teIF_Branch = 1 << 0,
  teIF_IndirectBranch = 1 << 1,
  teIF_Load = 1 << 2,
  teIF_Store = 1 << 3,
  teIF_Integer = 1 << 4,
  teIF_Float = 1 << 5,
  teIF_Arithmetic = 1 << 6,
  teIF_Annulled = 1 << 7,
  teIF_UncondTaken = 1 << 8,
  teIF_UncondNotTaken = 1 << 9,
  teIF_OnPage = 1 << 10,
  teIF_ModeSwitch = 1 << 11,
} temu_InstructionFlags;

typedef struct {
  int (*beginBlock)(void *Obj, uint64_t VA, uint64_t PA);
  int (*endBlock)(void *Obj, uint64_t VA, uint64_t PA);
  int (*beginInstr)(void *Obj, uint64_t VA, uint64_t PA, uint32_t Instr,
                    uint32_t Flags);
  // Arm: 0 = normal / not taken branch
  //      1 = taken conditional instruction
  int (*endInstr)(void *Obj, uint64_t VA, uint64_t PA, uint32_t Instr,
                  uint32_t Flags, int Arm);
} temu_InstrumenterIface;

#define TEMU_INSTRUMENTER_IFACE_TYPE "InstrumenterIface"
TEMU_IFACE_REFERENCE_TYPE(temu_Instrumenter);

#endif // ! TEMU_INSTRUMENTER_H
