//===-- temu-c/Simulator.h - Simulator Singleton API ------------*- C++ -*-===//
//
// TEMU: The Terma Emulator
// (c) Terma 2016
// Authors: Mattias Holm <maho (at) terma.com>
//
//===----------------------------------------------------------------------===//

#ifndef TEMU_SUPPORT_SIMULATOR_H
#define TEMU_SUPPORT_SIMULATOR_H

//------------------------------------------------------------------------------
// WARNING!!! WARNING!!!    EXPERIMENTAL API   WARNING!!!  WARNING!!! WARNING!!!
//
// BE WARNED THAT THIS API IS EXPERIMENTAL, UNSTABLE AND NOT WELL
// TESTED.  IN PARTICULAR, ASSUME THAT DOCUMENTATION FOR THE API IS
// NOT UP TO DATE AND IT MAY CRASH, CAUSE DEADLOCKS OR BEHAVE BADLY IN
// OTHER WAYS ON OCCASIONS. FURTHER THE API IS SUBJECT TO CHANGE
// WITHOUT NOTICE.
//------------------------------------------------------------------------------

#include "temu-c/Support/Attributes.h"
#include "temu-c/Target/Cpu.h"

#include <stdint.h>
#include <stdbool.h>

#ifdef __cplusplus
extern "C" {
#endif

/*!
 * Set the quanta of the simulator in nanoseconds.
 *
 * The quanta is used when scheduling processors. It is essentially
 * identical to the machine class quanta property. Effectively, each
 * processor in the system will synchronise with the other processors
 * and ensure that their timers are identical (within a small
 * tolerance, as a CPU will run until the quanta end plus the slack
 * introduced by the last instruction). Note that the quantas are set
 * in nanoseconds as cycles are inappropriate when multiple processors
 * of different clocks frequencies are used. Note that runtime quantas
 * are adapted based on when synchronised events occur, and for the
 * last quanta when running the processor.
 *
 * \param Ns Nanoseconds of quanta
 */
TEMU_API void temu_simSetQuanta(int64_t Ns);

/*!
 * Get current time of simulator in nanoseconds.
 *
 * The simulator time will in normal cases be adjusted to be a
 * multiple of the quanta, but as quantas are dynamically adjusted
 * when executing synchronised events, and when the last quanta is
 * executed, you cannot rely on the time being a multiple of the
 * quanta.
 *
 * \result Current time in nanoseconds
 */
TEMU_API int64_t temu_simGetTime(void);

/*!
 * Run the simulator for the given amount of nanoseconds
 *
 * \param Nanos Nanoseconds to run the simulator
 * \param Detatched Set to true to run in a separate thread.
 * \result Exit reason of the last executed CPU. Under normal
 * conditions this is teCER_Normal, but may be other values in-case
 * breakpoints are hit.
 */
temu_CpuExitReason temu_simRunNanos(int64_t Nanos, bool Detatched);
temu_CpuExitReason temu_simRunCycles(int Cpu, int64_t Cycles, bool Detatched);
temu_CpuExitReason temu_simRunSecs(double Secs, bool Detatched);

/*!
 * Return 0 if sim is not running, non-zero otherwise.
 */
int temu_simIsRunning(void);


/*!
 * Step the CPU for a number of steps.
 *
 * The function will step the CPU with the provided CPU number for a
 * number of steps. A step is defined to be equal to one instruction,
 * or one time advancement in case the CPU is in idle mode or halted.
 *
 * Note that it is not necessarily so that the CPU id in question is
 * still the current CPU when the function returns. Other CPUs may for
 * example hit a breakpoint or enter halt/error-mode.
 *
 * \param Cpu CPU number to step
 * \param Steps Number of steps to run.
 * \result Exit reason of the last executed CPU.
 */

TEMU_API temu_CpuExitReason temu_simStep(int Cpu, uint64_t Steps);

/*!
 * Get the id of the current CPU for the simulator.
 *
 * \result CPU id of the current CPU
 */
TEMU_API int temu_simGetCurrentCpu(void);

/*!
 * Get the number of CPUs in the simulator.
 *
 * \result Number of CPUs
 */
TEMU_API int temu_simGetCpuCount(void);

/*!
 * Stop the simulator.
 *
 * Returns after the simulator has stopped.
 */
void temu_simStop(void);


/*!
 * Execute a callback
 *
 * The function will run the callback in the current thread if the
 * simulator is not running, otherwise it will post it as a
 * thread-safe callback which will be called a bit later during
 * emulator event processing.
 *
 * \param Func Function to invoke
 * \param Data Pointer to pass to Func
 */
void temu_simRunCallback(void (*Func)(void *), void *Data);

/*!
 * Get the last exit reason for the simulator
 *
 * This can be used to query the last exit result for the run and step
 * functions.
 *
 * \result Exit reason from last call.
 */
temu_CpuExitReason temu_simGetExitReason();


#ifdef __cplusplus
}
#endif

#endif // !TEMU_SUPPORT_SIMULATOR_H
