//===-- temu-c/PowerPC.h - PowerPC Architecture Interfaces --------*- C++
//-*-===//
//
// TEMU: The Terma Emulator
// (c) Terma 2018
// Authors: Mattias Holm <maho (at) terma.com>
//
//===----------------------------------------------------------------------===//

#ifndef TEMU_POWERPC_H
#define TEMU_POWERPC_H

#include "temu-c/Memory/Memory.h"
#include "temu-c/Support/Objsys.h"
#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

/*!
  Interface for PowerPC specific functionality
 */
typedef struct temu_PowerPCIface {
  // Should go through property instead
  uint64_t (*getXER)(const void *Cpu);
  void (*setXER)(void *Cpu, uint64_t Value);
  uint32_t (*getCR)(const void *Cpu);
  void (*setCR)(void *Cpu, uint32_t Value);
  uint64_t (*getMSR)(const void *Cpu);
  void (*setMSR)(void *Cpu, uint64_t Value);
  uint64_t (*getReserveAddress)(const void *Obj);
  void (*setReserveAddress)(const void *Obj, uint64_t address);
  int (*isReservationBitSet)(const void *Obj);
  void (*clearAddressReservation)(const void *Obj);
  uint32_t (*getCTR)(const void *Cpu);
  void (*setCTR)(void *Cpu, uint32_t Value);
  uint64_t (*getLR)(const void *Cpu);
  void (*setLR)(void *Cpu, uint64_t Value);
  uint32_t (*getFPSCR)(const void *Cpu);
  void (*setFPSCR)(void *Cpu, uint32_t Value);
  void (*installEAResource)(void *Obj, unsigned RID,
                            temu_MemAccessIfaceRef Device);
} temu_PowerPCIface;

#define TEMU_POWERPC_IFACE_TYPE "PowerPCIface"
TEMU_IFACE_REFERENCE_TYPE(temu_PowerPC);

#ifdef __cplusplus
}
#endif

#endif /* ! TEMU_POWERPC_H */
