#include <stdint.h>
#include <stdio.h>
#include <string.h>

#include "temu-c/Bus/PCI.h"
#include "temu-c/Models/IrqController.h"
#include "temu-c/Support/Attributes.h"
#include "temu-c/Support/Logging.h"
#include "temu-c/Support/Objsys.h"

typedef struct {
  temu_PCIDevice Super;
  temu_PCIBridgeIfaceRef Bridge;
  temu_IrqCtrlIfaceRef IRQ;
} PciDevice;

static void
startSelfTest(temu_Object *Obj)
{
}

static temu_PCIConfig *
getPciConfig(temu_Object *Obj)
{
  PciDevice *P = (PciDevice *)Obj;
  return &P->Super.Conf;
}
// Writes to config space forwarded here if outside the standard config space

static void
writeConfig(temu_Object *Obj, uint32_t offset, uint32_t value)
{
  // Write config is called whenver we write to a config register not in the
  // standard PCI config header (i.e. offset 0x3c and onwards).
  // PciDevice *P = (PciDevice *)Obj;
  // switch (MT->Offset) {
  // case 0x3c:
  //  break;
  //}
}
static uint32_t
readConfig(temu_Object *Obj, uint32_t offset)
{
  temu_logDebug(Obj, "Read from configuration area with offset 0x%x ",
                (unsigned)offset);
  return 0;
}
static uint64_t
getPciBarSize(temu_Object *Obj, unsigned Bar)
{
  switch (Bar) {
  case 0:
    return 4096;
  case 1:
  case 2:
  case 3:
  case 4:
  case 5:
    return 4096;
  }

  return 0;
}

static uint64_t
getPciExpansionROMSize(temu_Object *Obj)
{
  return 0;
}

static temu_PCIDeviceIface PCIDevIface = {
    startSelfTest, getPciConfig,  writeConfig,
    readConfig,    getPciBarSize, getPciExpansionROMSize};

static void
ioRead(void *Obj, temu_MemTransaction *MT, int BAR)
{
  assert(BAR == 5);
}
static void
ioWrite(void *Obj, temu_MemTransaction *MT, int BAR)
{
  assert(BAR == 5);
}

static void
memRead(void *Obj, temu_MemTransaction *MT, int BAR)
{
  assert(BAR < 5);
}

static void
memWrite(void *Obj, temu_MemTransaction *MT, int BAR)
{
  assert(BAR < 5);
}

#define MEM_ACCESS(I, IM)                                                      \
  static void memAccessWrite##I(void *Obj, temu_MemTransaction *MT)            \
  {                                                                            \
    IM##Write(Obj, MT, I);                                                     \
  }                                                                            \
  static void memAccessRead##I(void *Obj, temu_MemTransaction *MT)             \
  {                                                                            \
    IM##Read(Obj, MT, I);                                                      \
  }

MEM_ACCESS(0, mem)
MEM_ACCESS(1, mem)
MEM_ACCESS(2, mem)
MEM_ACCESS(3, mem)
MEM_ACCESS(4, mem)
MEM_ACCESS(5, io)

static temu_MemAccessIface MemAccessIfaces[6] = {
    {NULL, memAccessRead0, memAccessWrite0, NULL, NULL},
    {NULL, memAccessRead1, memAccessWrite1, NULL, NULL},
    {NULL, memAccessRead2, memAccessWrite2, NULL, NULL},
    {NULL, memAccessRead3, memAccessWrite3, NULL, NULL},
    {NULL, memAccessRead4, memAccessWrite4, NULL, NULL},
    {NULL, memAccessRead5, memAccessWrite5, NULL, NULL},
};

static void
expRomRead(void *Obj, temu_MemTransaction *MT)
{
}

static void
expRomWrite(void *Obj, temu_MemTransaction *MT)
{
  return; // ROM, ignore
}
static temu_MemAccessIface ExpRomAccessIface = {NULL, expRomRead, expRomWrite,
                                                NULL, NULL};

static void *
create(const char *Name, int Argc TEMU_UNUSED,
       const temu_CreateArg *Argv TEMU_UNUSED)
{
  PciDevice *Obj = malloc(sizeof(PciDevice));
  memset(Obj, 0, sizeof(PciDevice));

  temu_pciSetDeviceId(&Obj->Super.Conf, 0x0000);
  temu_pciSetVendorId(&Obj->Super.Conf, 0x0000);
  temu_pciSetClassCode(&Obj->Super.Conf, tePciBc_Pre);
  temu_pciSetRevId(&Obj->Super.Conf, 0x01);
  temu_pciSetBist(&Obj->Super.Conf, 0x00);       // Does not support BIST
  temu_pciSetHeaderType(&Obj->Super.Conf, 0x00); // Normal header

  Obj->Super.Conf.BAR[0] = 0; // Mem space
  Obj->Super.Conf.BAR[1] = 0; // Mem space
  Obj->Super.Conf.BAR[2] = 0; // Mem space
  Obj->Super.Conf.BAR[3] = 0; // Mem space
  Obj->Super.Conf.BAR[4] = 0; // Mem space
  Obj->Super.Conf.BAR[5] = 1; // IO space

  return Obj;
}

void
dispose(void *Obj)
{
  free(Obj);
}

TEMU_PLUGIN_INIT
{
  temu_Class *Cls = temu_registerClass("PciDevice", create, dispose);
  temu_pciDeviceRegister(Cls);

  temu_addInterfaceReference(Cls, "bridge", offsetof(PciDevice, Bridge),
                             TEMU_PCI_BRIDGE_IFACE_TYPE, 1, 0, NULL, NULL,
                             "PCI bridge reference");

  temu_addInterfaceReference(Cls, "irqCtrl", offsetof(PciDevice, IRQ),
                             TEMU_IRQ_CTRL_IFACE_TYPE, 1, 0, NULL, NULL,
                             "IRQ controller reference");

  temu_addInterface(Cls, "PCIDevIface", TEMU_PCI_DEVICE_IFACE_TYPE,
                    &PCIDevIface, 0, "PCI device interface");

  temu_addInterfaceArray(Cls, "MemAccessIface", TEMU_MEM_ACCESS_IFACE_TYPE,
                         &MemAccessIfaces[0], 6, sizeof(temu_MemAccessIface),
                         "Memory access interfaces per BAR");
  temu_addInterface(Cls, "ExpROMAccessIface", TEMU_MEM_ACCESS_IFACE_TYPE,
                    &ExpRomAccessIface, 0, "Exp ROM access interface");
}
